#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

#define LINEWIDTH 1
#define WINDOWNAME "NESL Graphics Display"
#define ICONNAME "NESLG"

Display *display;
Window win;

Display *getdisplay(char *displayname)
{
  Display *disp;
  if ((disp = XOpenDisplay(displayname)) == NULL){
    fprintf(stderr, "Cannot connect to Xserver %s\n",
	    XDisplayName(NULL));
    exit(-1);
  }
  return disp;
}

Window Create_Window(int x, int y, int width, int height, char *displayname)
{
  int screen;
  Window win;
  unsigned long mask, fore_pix, back_pix;
  XSizeHints	size_hints;
  XWMHints wm_hints;
  XClassHint class_hints;
  XTextProperty wp, ip;
  static char *windowname;
  static char *iconname;
  XSetWindowAttributes winattr;
  XEvent ev;
    
  windowname = (char *)malloc(strlen(WINDOWNAME)+1);
  strcpy(windowname, WINDOWNAME);
  iconname = (char *)malloc(strlen(ICONNAME)+1);
  strcpy(iconname, ICONNAME);

  display = getdisplay(displayname);

  screen = DefaultScreen(display);
  fore_pix = WhitePixel(display, screen);
  back_pix = BlackPixel(display, screen);

  win = XCreateSimpleWindow (display, RootWindow(display,screen),
			     x,y, width, height,
			     3,	/* border width */
			     fore_pix,back_pix);
  size_hints.flags = USPosition | USSize;

  wm_hints.initial_state = NormalState;
  wm_hints.flags = StateHint;
    
  class_hints.res_name = "program name";
  class_hints.res_class = "program class name";

  if (XStringListToTextProperty(&windowname, 1, &wp) == 0 ||
      XStringListToTextProperty(&iconname, 1, &ip) == 0 ){
    fprintf(stderr,"name allocation failed\n");
    exit(-1);
  }
  XSetWMProperties(display, win, &wp, &ip, NULL, 0,
		   &size_hints, &wm_hints, &class_hints);

  winattr.backing_store = Always;
  winattr.bit_gravity = NorthWestGravity;
  mask = CWBackingStore | CWBitGravity;

  XSelectInput(display, win, ExposureMask);
  XMapWindow(display,win);
  do {
    XNextEvent(display,&ev);
    if (ev.type == Expose)
      break;
  } while (1);			/* wait until the window is exposed */
  XChangeWindowAttributes(display, win, mask, &winattr);
  return win;
}

GC linedef(Display *display)
{
  unsigned long mask;
  XGCValues values;
  int screen = DefaultScreen(display);

  mask = GCFunction | GCForeground | GCBackground | GCLineWidth;
  values.function = GXcopy;
  values.foreground = WhitePixel(display, screen);
  values.background = BlackPixel(display, screen);
  values.line_width = LINEWIDTH;

  return XCreateGC(display, RootWindow(display, screen), mask, &values);
}

void exit_on_mouse()
{
  XEvent ev;
  XSelectInput(display, win, ButtonPressMask);
  XMapWindow(display,win);
  XNextEvent(display,&ev);
  exit(0);
}

void nextline()
{
  char c = getchar();
  do { c = getchar(); }
  while (c == ' ' || c == ')' || c == ']' || c == '\n');
  if (c == EOF) exit_on_mouse();
}

readdrawpoints(Window win)
{
  XPoint *plist;
  int l,i,p;
  nextline();
  scanf("%d", &l);
  plist = (XPoint *) malloc(sizeof(XPoint)*l);
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].x = p;
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].y = p;
  XDrawPoints(display, win, linedef(display), plist, l, 0);
  XFlush(display);
}

readdrawlines(Window win)
{
  XPoint *plist;
  int l,i,p;
  nextline();
  scanf("%d", &l);
  plist = (XPoint *) malloc(sizeof(XPoint)*l);
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].x = p;
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].y = p;
  XDrawLines(display, win, linedef(display), plist, l, 0);
  XFlush(display);
}

readdrawsegments(Window win)
{
  XSegment *plist;
  int l,i,p;
  nextline();
  scanf("%d", &l);
  plist = (XSegment *) malloc(sizeof(XSegment)*l);
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].x1 = p;
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].y1 = p;
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].x2 = p;
  nextline();
  for(i = 0; scanf("%d", &p); i++) plist[i].y2 = p;
  XDrawSegments(display, win, linedef(display), plist, l);
  XFlush(display);
}

int gettype()
{
  int i;
  nextline();
  scanf("%d", &i);
  return i;
}

main() {
  int x0,y0,width,height;
  char displayname[100];

  scanf("%s",displayname);
  getchar();
  getchar();
  scanf("%d", &x0);
  scanf("%d", &y0);
  scanf("%d", &width);
  scanf("%d", &height);
  win = Create_Window(x0,y0,width,height,displayname);
  do {
    int command = gettype();
    if (command == 0) readdrawpoints(win);
    else if (command == 1) readdrawlines(win); 
    else if (command == 2) readdrawsegments(win);
    else exit_on_mouse();
  } while(1);
}
